!***********************************************************************
! program ambig                                                        *
!    Resolve the 180 degree ambiguity in the transverse component of   *
! the magnetic field using the Minimum Energy approach first described *
! by Metcalf (1994, Solar Phys., 155, 235). The code reads in data from*
! either a FITS or text file, calculates derivatives for a potential   *
! field, then uses simulated annealing to minimize the sum of the      *
! absolute value of the divergence plus the absolute value of the      *
! vertical current density.                                            *
! For pixels with a transverse field strength below a given threshold  *
! the ambiguity is resolved using an acute angle to nearest neighbors  *
! methods, based on that described in Canfield et al. (1993, ApJ,      *
! 411, 362).                                                           *
! The potential field is calculated directly from the line of sight    *
! component of the field on a uniform grid in image coordinates. This  *
! avoids having to interpolate to and from a uniform grid in           *
! helioplanar coordinates in order to use FFTs. In order to minimize   *
! edge effects, the code surrounds the observed field with an area of  *
! zero field, and uses a bi-cubic spline to smoothly interpolate from  *
! observed field to zero field.                                        *
!***********************************************************************

program ambig

!-----------------------------------------------------------------------
   use sizes
   use flags
   use maskvec
   use mgram_data
   use pad
   use point
   use pot_field
   use verbose

   implicit none

   integer :: nxp,nyp,i,j, argc
   real :: t1
   real :: athresh,bthresh
   character(len=256) :: filename, azfile, parfile
   external :: setup_OCBP_PF_dzh_4p,CalcE_OCBP_PF_dzh_4p,CalcDE_reconfig_OCBP_PF_dzh_4p
   external :: setup_spherical_PF_4p,CalcE_spherical_PF_4p,CalcDE_reconfig_spherical_PF_4p
   logical :: file_exists
!-----------------------------------------------------------------------
! --> Get parameter and azimuth output filenames from command line,
! --> if they are both available.
  argc = command_argument_count()       ! Number of command line args
  if (argc == 1) then
      call get_command_argument(1, parfile)
      azfile='azimuth.dat'
  else if (argc == 2) then
      call get_command_argument(1, parfile)
      call get_command_argument(2, azfile)
  else
      azfile='azimuth.dat'
      parfile='par'
  endif
  print*, 'parfile=',trim(parfile)
  print*, 'azfile=',trim(azfile)

! --> Read in some parameters which set options for the code. 

   call rd_param(parfile, filename,athresh,bthresh)
   if(iverb.ge.1) write(*,*) 'parameters read'

! --> Use filename from command line if given, superceding what is in
! --> the par file, and make sure the input file exists in any case.
   inquire(file=filename, exist=file_exists)
   if (.not. file_exists) then
      write(*,'(a)') 'Input file does not exist: '//adjustl(trim(filename))
      stop
   endif

! --> Read in magnetic field and pointing information. 
! --> irflag determines whether it is from a FITS file or not, and 
! --> whether the FITS file is from the Hinode SOT/SP pipeline. 

   if(irflag.eq.0) then
      call rd_field(filename)
   else if(irflag.eq.1) then
      call rd_fits(filename)
   else if(irflag.eq.2) then
      call rd_sotsp(filename)
   endif
   if(iverb.ge.1) write(*,*) 'field read'
!
! --> Determine box enclosing all above-threshold pixels. 
!
   call boxit(athresh)
   if(iverb.ge.1) write(*,*) 'boxit done'
!
! --> Set up mask array determining how each pixel will be treated
!
   call setup_mask(athresh,bthresh)
   if(iverb.ge.1) write(*,*) 'mask constructed'
!
   select case (igflag)
!
! igflag=1: the divergence and vertical current density are
! calculated with the helioplanar approximation
!
      case(1)
!
! --> Construct coordinate transformation matrices. 
!
         call transform(theta,phi)
         if(iverb.ge.1) write(*,*) 'transform calculated'
!
! --> Pad the input array with zeros.
! --> This reduces the impact of the periodic boundary conditions. 
!
         call buffer(Bz,nxp,nyp)
         if(iverb.ge.1) write(*,*) 'buffering done'
!
! --> Smooth the transition to zero field. 
! --> This reduces ringing if the field is non-zero at the boundaries.
!
         if(nap.ge.1) call smooth()
         if(iverb.ge.1) write(*,*) 'apodizing done'
!
! --> Calculate potential field (and derivatives) using FFTs. 
!
         allocate(Bpix(nxp,nyp),Bpiy(nxp,nyp),dBpzdz(nxp,nyp))
         call potential(nxp,nyp)
         do i=1,nx
            do j=1,ny
               dBzdz(i,j)=dBpzdz(npad+i,npad+j)
            enddo
         enddo
         deallocate(dBpzdz)
         if(iverb.ge.1) write(*,*) 'potential field done'
!
! --> Perform acute angle to potential field ambiguity resolution on
! --> pixels below a given threshold. 
!
         if(ipflag.eq.1) then
            call pacute(bthresh)
            if(iverb.ge.1) write(*,*) 'potential field ambiguity resolution done'
         endif
         deallocate(Bpix,Bpiy)
!
! --> Global minimization of "energy" to resolve ambiguity. 
!
         call global(setup_OCBP_PF_dzh_4p,CalcE_OCBP_PF_dzh_4p,CalcDE_reconfig_OCBP_PF_dzh_4p)
!
! igflag=2: the divergence and vertical current density are
! calculated in spherical coordinates
!
      case(2)

! --> Calculate potential field (and derivatives) using tiling.
!
         call tile
         if(iverb.ge.1) write(*,*) 'tiling done'
!
! --> Make sure mask does not include any pixels for which the potential
! --> field derivative does not exist.
!
         do i=1,nx
            do j=1,ny
               if(mask(i,j).eq.2.and.tmask(i,j).eq.0) then
                  mask(i,j)=1
               endif
            enddo
         enddo
!
         deallocate(tmask)
!
! --> Global minimization of "energy" to resolve ambiguity. 
!
         call global(setup_spherical_PF_4p,CalcE_spherical_PF_4p,CalcDE_reconfig_spherical_PF_4p)
!
      case default
         if(iverb.ge.1) write(*,*) 'wrong geometry'
         stop
   end select
   if(iverb.ge.1) write(*,*) 'optimization done'
!
! --> Perform acute angle to nearest neighbor ambiguity resolution on
! --> pixels below a given threshold. 
!
   call nacute6
   if(iverb.ge.1) write(*,*) 'smoothing done'

! --> Write out the azimuth array
   call write_field(azfile)

!
! --> If the data came from a FITS file, update it; otherwise, write out
! --> the azimuth array. 
!
 ! if(irflag.eq.0) then
 !    call write_field(azfile)
 ! else if(irflag.eq.1) then
 !    call update_fits(filename)
 ! else if(irflag.eq.2) then
 !    call write_field(azfile)
 ! endif
   if(iverb.ge.1) write(*,*) 'results written'
call cpu_time(t1)
write(*,*) 'CPU time',t1

end program ambig
