!***********************************************************************
! subroutine rd_fits                                                   *
!    Reads in magnetic field data plus pointing information from a     *
! FITS file. Based on routines given in cookbook.f supplied with       *
! FITSIO.                                                              *
!***********************************************************************

subroutine rd_fits(filename)

!-----------------------------------------------------------------------
   use sizes
   use bobs
   use constant
   use disk_center
   use ephemeris
   use fits
   use flags
   use mgram_data
   use pix_size
   use point
   use spherical_position

   implicit none

   integer :: i,j,i1d
   integer :: status,unit,readwrite,blocksize
   integer :: firstpix,group,nfound,npixels,narray
   real :: nullval,phase,dtor,xx,yy,y2
   real,dimension(:),allocatable :: bl1d,bt1d,ba1d,bf1d
   logical :: anynull
   character(len=90) :: filename
   character(len=80) :: record,comment
!-----------------------------------------------------------------------
! --> The status parameter must always be initialized.
   status=0

! --> Get an unused Logical Unit Number to use to open the FITS file.
   call ftgiou(unit,status)

! --> Open the FITS file, with read-only access.  The returned blocksize
! --> parameter is obsolete and should be ignored. 
   readwrite=0
   call ftopen(unit,filename,readwrite,blocksize,status)
   if(status.ne.0) then
      write(*,*) 'error opening FITS file'
      call printerror(status)
      stop
   endif

! --> Determine the size of the image.
   call ftgknj(unit,'NAXIS',1,3,naxes,nfound,status)
   if(status.ne.0) then
      write(*,*) 'error reading NAXIS',nfound,naxes(1),naxes(2)
      call printerror(status)
      stop
   endif
   if(nfound.ne.3) stop '*** Unexpected number of axes found ***'

   nx=naxes(1)
   ny=naxes(2)

! --> Allocate memory.
   allocate(bl(nx,ny),bt(nx,ny),ba(nx,ny))
   allocate(Bx(nx,ny),By(nx,ny),Bz(nx,ny),dBzdz(nx,ny))

! --> Get the ephemeris information from the keywords. 
! --> b=solar b-angle
! --> p=solar p-angle
! --> radius= solar radius

   call ftgkye(unit,radkey,radius,comment,status)
   if(status.ne.0) then
      write(*,*) 'error encountered while looking for radius'
      call printerror(status)
      radius=959.689
      write(*,*) 'using default value of radius=',radius
      status=0
   endif

   call ftgkye(unit,pkey,p0,comment,status)
   if(status.ne.0) then
      write(*,*) 'error encountered while looking for p-angle'
      call printerror(status)
      p0=0.
      write(*,*) 'using default value of p-angle=',p0
      status=0
   endif

   call ftgkye(unit,bkey,b,comment,status)
   if(status.ne.0) then
      write(*,*) 'error encountered while looking for b-angle'
      call printerror(status)
      b=0.
      write(*,*) 'using default value of b-angle=',b
      status=0
   endif
write(*,*) 'ephemeris',radius,p0,b

! --> Convert to radians, if necessary.
   if(ipunit.eq.1) then
      p0=p0*pi/180.
      b=b*pi/180.
   endif

! --> Get the pixel size from the keywords.

   call ftgkye(unit,pxkey,dxi,comment,status)
   call ftgkye(unit,pykey,dyi,comment,status)
write(*,*) 'pixel size',dxi,dyi

! --> Get the pointing information from the keywords.
   if(igflag.eq.2) then
! --> Try reading in latitude/longitude.
      call ftgkye(unit,cmdkey,theta,comment,status)
      !if(status.ne.0) then
         !write(*,*) 'error encountered while looking for longitude'
         !call printerror(status)
         !theta=0.
         !write(*,*) 'using default value of longitude=',theta
         !status=0
      !endif

      call ftgkye(unit,latkey,phi,comment,status)
      !if(status.ne.0) then
         !write(*,*) 'error encountered while looking for latitude'
         !call printerror(status)
         !phi=0.
         !write(*,*) 'using default value of latitude=',phi
         !status=0
      !endif

! --> Convert to radians, if necessary.
      if(ipunit.eq.1) then
         theta=theta*pi/180.
         phi=phi*pi/180.
      endif

      if(status.ne.0) then
! --> Try reading in the position of solar center instead.
         write(*,*) 'error encountered while looking for latitude/longitude'
         call printerror(status)
         write(*,*) 'checking for x-center/y-center instead'
         status=0

         call ftgkye(unit,xcenkey,xcen,comment,status)
         if(status.ne.0) then
            write(*,*) 'error encountered while looking for x-center'
            call printerror(status)
            xcen=0.
            write(*,*) 'using default value of xcen=',xcen
            status=0
         endif

         call ftgkye(unit,ycenkey,ycen,comment,status)
         if(status.ne.0) then
            write(*,*) 'error encountered while looking for y-center'
            call printerror(status)
            ycen=0.
            write(*,*) 'using default value of ycen=',ycen
            status=0
         endif

! --> Convert location of disk center to latitude/longitude of center of FOV
! --> (assuming xcen, ycen are given in the same units as the radius). 
         xx=(xcen*cos(p0)+ycen*sin(p0))/radius
         yy=(-xcen*sin(p0)+ycen*cos(p0))/radius

         phi=asin(yy*cos(b)+sin(b)*sqrt(1.-xx**2-yy**2))
         theta=atan(xx*sin(b)/(sin(phi)*cos(b)-yy))
write(*,*) 'lat choices/lon',phi,asin(yy*cos(b)-sin(b)*sqrt(1.-xx**2-yy**2)),theta
      endif

! --> Generate colat/lon for each pixel.
      call colatlon
      write(*,*) 'xcen, ycen',xcen,ycen
      write(*,*) 'lat, lon',phi,theta

   else
      call ftgkye(unit,cmdkey,theta,comment,status)
      if(status.ne.0) then
         write(*,*) 'error encountered while looking for longitude'
         call printerror(status)
         theta=0.
         write(*,*) 'using default value of longitude=',theta
         status=0
      endif

      call ftgkye(unit,latkey,phi,comment,status)
      if(status.ne.0) then
         write(*,*) 'error encountered while looking for latitude'
         call printerror(status)
         phi=0.
         write(*,*) 'using default value of latitude=',phi
         status=0
      endif

! --> Convert to radians, if necessary.
      if(ipunit.eq.1) then
         theta=theta*pi/180.
         phi=phi*pi/180.
      endif
write(*,*) 'lat/lon',phi,theta
   endif

! --> Initialize variables for reading field. 
   narray=naxes(1)*naxes(2)
   npixels=narray*naxes(3)
   group=1
   nullval=-999.

! --> Allocate memory for 1-d arrays for reading. 
   allocate(bl1d(narray),bt1d(narray),ba1d(narray),bf1d(narray))

! --> Different steps required if field is specified as a magnitude plus
! --> two angles versus two components plus one angle. 

   phase=0.5*float(iaflag)*pi
   if(ibflag.eq.0) then

! --> Read in line of sight component.
      firstpix=1+narray*(iblong-1)
      call ftgpve(unit,group,firstpix,narray,nullval,bl1d,anynull,status)

      do i=1,nx
         do j=1,ny
            bl(i,j)=bl1d(i+(j-1)*nx)
         enddo
      enddo

! --> Read in transverse component.
      firstpix=1+narray*(ibtrans-1)
      call ftgpve(unit,group,firstpix,narray,nullval,bt1d,anynull,status)

      do i=1,nx
         do j=1,ny
            bt(i,j)=bt1d(i+(j-1)*nx)
         enddo
      enddo

! --> Read in fill fraction, if present, and multiply field strength by fill
! --> fraction.
      if(ibfill.gt.0) then
         firstpix=1+narray*(ibfill-1)
         call ftgpve(unit,group,firstpix,narray,nullval,bf1d,anynull,status)
         do i=1,nx
            do j=1,ny
               bl(i,j)=bl(i,j)*bf1d(i+(j-1)*nx)
               bt(i,j)=bt(i,j)*bf1d(i+(j-1)*nx)
            enddo
         enddo
      endif

! --> Read in azimuthal angle.
      firstpix=1+narray*(ibazim-1)
      call ftgpve(unit,group,firstpix,narray,nullval,ba1d,anynull,status)

! --> Convert to radians if necessary. 
      if(iaunit.eq.1) then
         dtor=pi/180.
         ba1d=ba1d*dtor
      endif

      do i=1,nx
         do j=1,ny
            ba(i,j)=ba1d(i+(j-1)*nx)
! --> Calculate Cartesian (image) components.
            Bx(i,j)=bt(i,j)*cos(ba(i,j)+phase)
            By(i,j)=bt(i,j)*sin(ba(i,j)+phase)
            Bz(i,j)=bl(i,j)
         enddo
      enddo

   else

! --> Read in magnitude.
      firstpix=1+narray*(iblong-1)
      call ftgpve(unit,group,firstpix,narray,nullval,bl1d,anynull,status)

! --> Read in fill fraction, if present, and multiply field strength by fill
! --> fraction.
      if(ibfill.gt.0) then
         firstpix=1+narray*(ibfill-1)
         call ftgpve(unit,group,firstpix,narray,nullval,bf1d,anynull,status)
         do i=1,nx
            do j=1,ny
               bl(i,j)=bl(i,j)*bf1d(i+(j-1)*nx)
               bt(i,j)=bt(i,j)*bf1d(i+(j-1)*nx)
            enddo
         enddo
      endif

! --> Read in inclination angle.
      firstpix=1+narray*(ibtrans-1)
      call ftgpve(unit,group,firstpix,narray,nullval,bt1d,anynull,status)

! --> Read in azimuthal angle.
      firstpix=1+narray*(ibazim-1)
      call ftgpve(unit,group,firstpix,narray,nullval,ba1d,anynull,status)

! --> Convert to radians if necessary. 
      if(iaunit.eq.1) then
         dtor=pi/180.
         bt1d=bt1d*dtor
         ba1d=ba1d*dtor
      endif

! --> Transform to two components. 
      if(incflag.eq.0) then
         do i=1,nx
            do j=1,ny
               i1d=i+(j-1)*nx
               bl(i,j)=bl1d(i1d)*cos(bt1d(i1d))
               bt(i,j)=bl1d(i1d)*sin(bt1d(i1d))
               ba(i,j)=ba1d(i1d)
! --> Calculate Cartesian (image) components.
               Bx(i,j)=bt(i,j)*cos(ba(i,j)+phase)
               By(i,j)=bt(i,j)*sin(ba(i,j)+phase)
               Bz(i,j)=bl(i,j)
            enddo
         enddo
      else
         do i=1,nx
            do j=1,ny
               i1d=i+(j-1)*nx
               bl(i,j)=bl1d(i1d)*sin(bt1d(i1d))
               bt(i,j)=bl1d(i1d)*cos(bt1d(i1d))
               ba(i,j)=ba1d(i1d)
! --> Calculate Cartesian (image) components.
               Bx(i,j)=bt(i,j)*cos(ba(i,j)+phase)
               By(i,j)=bt(i,j)*sin(ba(i,j)+phase)
               Bz(i,j)=bl(i,j)
            enddo
         enddo
      endif

   endif

! --> Deallocate memory.
   deallocate(bl1d,bt1d,ba1d,bf1d)

! --> The FITS file must always be closed before exiting the program. 
! --> Any unit numbers allocated with FTGIOU must be freed with FTFIOU.
   call ftclos(unit, status)
   call ftfiou(unit, status)

! --> Check for any error, and if so print out error messages.
   if(status.gt.0) call printerror(status)

end subroutine rd_fits
