!***********************************************************************
! subroutine rd_sotsp                                                  *
!    Reads in magnetic field data plus pointing information from a     *
! FITS file, in the format used by Hinode SOT/SP from NAOJ. Based on   *
! on routines given in cookbook.f supplied with FITSIO.                *
!***********************************************************************

subroutine rd_sotsp(filename)

!-----------------------------------------------------------------------
   use sizes
   use bobs
   use constant
   use disk_center
   use ephemeris
   use fits
   use flags
   use mgram_data
   use pix_size
   use point
   use spherical_position

   implicit none

   integer :: i,j,i1d
   integer :: status,unit,readwrite,blocksize
   integer :: firstpix,group,nfound,narray
   integer :: colnum,irow,felem,nelems,naxis,maxdim
   integer :: datacode,repeat,width
   integer :: hdutype,nlimb
   real :: nullval,dtor,phase,cimean,xx,yy,y2
   real,dimension(:),allocatable :: bs1d,bi1d,ba1d,sl1d,ci1d
   real,parameter :: cifrac=0.05
   logical :: anynull
   character(len=90) :: filename
   character(len=80) :: record,comment
!-----------------------------------------------------------------------
! --> The status parameter must always be initialized.
   status=0

! --> Get an unused Logical Unit Number to use to open the FITS file.
   call ftgiou(unit,status)

! --> Open the FITS file, with read-only access.  The returned blocksize
! --> parameter is obsolete and should be ignored. 
   readwrite=0
   call ftopen(unit,filename,readwrite,blocksize,status)

! --> Get the pixel size from the keywords.

   call ftgkye(unit,'XSCALE',dxi,comment,status)
   call ftgkye(unit,'YSCALE',dyi,comment,status)
   if(status.ne.0) then
      write(*,*) 'error encountered while looking for pixel size'
      call printerror(status)
      stop
   else
      write(*,*) 'pixel size',dxi,dyi
   endif

! --> Get the location of the FOV relative to disk center. 

   call ftgkye(unit,'XCEN',xcen,comment,status)
   call ftgkye(unit,'YCEN',ycen,comment,status)
   if(status.ne.0) then
      write(*,*) 'error encountered while looking for xcen,ycen'
      call printerror(status)
      stop
   else
      write(*,*) 'image position',xcen,ycen
   endif

! --> Get the ephemeris data from the keywords.

   !call ftgkye(unit,'SOLAR_RA',radius,comment,status)
   !call ftgkye(unit,'P_ANGLE',p0,comment,status)
   !call ftgkye(unit,'B_ANGLE',b,comment,status)
   !if(status.ne.0) then
   !   write(*,*) 'error encountered while looking for ephemeris'
   !   call printerror(status)
   !   stop
   !else
   !   write(*,*) 'ephemeris',radius,p0,b
   !endif
   radius=960.
   p0=0.
   b=0.

! --> Convert to radians, if necessary.

   p0=p0*pi/180.
   b=b*pi/180.

! --> Determine latitude and longitude from this. 
! --> theta=central meridian angle
! --> phi=latitude
! XXX include p and b angles in this
   theta=atan(xcen/sqrt(radius**2-xcen**2-ycen**2))
   phi=asin(ycen/radius)
   !write(*,*) theta,phi

! --> Jump straight to the HDU containing the field strength.

   call ftmahd(unit,jfs,hdutype,status)

! --> Check this HDU exists.

   if(status.ne.0) then
      write(*,*) 'error encountered while looking for field strength'
      call printerror(status)
      stop
   endif

! --> Have to treat this differently depending on whether it's an image 
! --> extension (hdutype=0) or a binary table extension (hdutype=2).

   if(hdutype.eq.0) then

! --> Check this is the field strength.
      call ftgkys(unit,'PAR_TYPE',record,comment,status)
      if(status.eq.0) then
         if(record.ne.'Field strength') then
            write(*,*) 'unexpected PAR_TYPE for field strength'
         else
            write(*,*) 'correct PAR_TYPE for field strength'
         endif
      else
         write(*,*) 'error encountered while getting field strength PAR_TYPE'
         call printerror(status)
         write(*,*) 'checking for EXTNAME instead'
         status=0
         call ftgkys(unit,'EXTNAME',record,comment,status)
         if(status.eq.0) then
            if(record.ne.'FieldStrength') then
               write(*,*) 'unexpected EXTNAME for field strength',record
            else
               write(*,*) 'correct EXTNAME found'
               write(*,*) ''
            endif
         else
            write(*,*) 'error encountered while getting field strength EXTNAME'
            call printerror(status)
         endif
      endif
      status=0

! --> Determine the size of the extension.
      call ftgknj(unit,'NAXIS',1,2,naxes,nfound,status)
      if(nfound.ne.2) then
         write(*,*) '*** Unexpected number of axes found while reading field strength ***',nfound
         write(*,*) naxes(1),naxes(2)
      endif

! --> Initialize variables for reading field strength. 
      narray=naxes(1)*naxes(2)
      group=1
      nullval=-999.
      firstpix=1

! --> Allocate memory for reading 1-d arrays.
      allocate(bs1d(narray),bi1d(narray),ba1d(narray),sl1d(narray),ci1d(narray))

! --> Read in field strength.
      call ftgpve(unit,group,firstpix,narray,nullval,bs1d,anynull,status)

   elseif(hdutype.eq.2) then

! --> Check this is the field strength.
      call ftgkys(unit,'EXTNAME',record,comment,status)
      if(record.ne.'FieldStrength') then
         write(*,*) 'unexpected EXTNAME for field strength',record
      endif

! --> Determine the size of the extension.
      maxdim=2
      colnum=1
      call ftgtcl(unit,colnum,datacode,repeat,width,status)
      call printerror(status)

      call ftgtdm(unit,colnum,maxdim,naxis,naxes,status)
      call printerror(status)

      if(naxis.ne.2) then
         write(*,*) '*** Unexpected number of axes found while reading field strength ***',naxis
         write(*,*) naxes(1),naxes(2)
      endif

      if(repeat.ne.naxes(1)*naxes(2)) then
         write(*,*) '*** Inconsistent row length found while reading field strength ***'
         write(*,*) repeat,naxes(1),naxes(2)
      endif

! --> Initialize variables for reading field strength. 
      irow=1
      felem=1
      nelems=repeat

! --> Allocate memory for reading 1-d arrays.
      narray=naxes(1)*naxes(2)
      allocate(bs1d(narray),bi1d(narray),ba1d(narray),sl1d(narray),ci1d(narray))

! --> Read in field strength.
      if(datacode.eq.42) then
         call ftgcve(unit,colnum,irow,felem,nelems,nullval,bs1d,anynull,status)
      else
         write(*,*) 'unexpected data type for field strength',datacode
         stop
      endif

   else
      write(*,*) 'unrecognized HDU type for field strength',hdutype
      stop
   endif

   nx=naxes(1)
   ny=naxes(2)

! --> If working in spherical geometry, generate colat/lon for each pixel.

   if(igflag.eq.2) call colatlon

! --> Allocate memory for remaining arrays.

   allocate(bl(nx,ny),bt(nx,ny),ba(nx,ny))
   allocate(Bx(nx,ny),By(nx,ny),Bz(nx,ny),dBzdz(nx,ny))

! --> Repeat for the HDU containing the field inclination.

   call ftmahd(unit,jfi,hdutype,status)

! --> Check this HDU exists.

   if(status.ne.0) then
     write(*,*) 'error encountered while looking for field inclination'
     call printerror(status)
     stop
   endif

! --> Have to treat this differently depending on whether it's an image 
! --> extension (hdutype=0) or a binary table extension (hdutype=2).

   if(hdutype.eq.0) then

! --> Check this is the field inclination.
      call ftgkys(unit,'PAR_TYPE',record,comment,status)
      if(status.eq.0) then
         if(record.ne.'Field inclination') then
            write(*,*) 'unexpected PAR_TYPE for field inclination'
         else
            write(*,*) 'correct PAR_TYPE for field inclination'
         endif
      else
         write(*,*) 'error encountered while getting field inclination PAR_TYPE'
         call printerror(status)
         write(*,*) 'checking for EXTNAME instead'
         status=0
         call ftgkys(unit,'EXTNAME',record,comment,status)
         if(status.eq.0) then
            if(record.ne.'FieldInclination') then
               write(*,*) 'unexpected EXTNAME for field inclination',record
            else
               write(*,*) 'correct EXTNAME found'
               write(*,*) ''
            endif
         else
            write(*,*) 'error encountered while getting field inclination EXTNAME'
            call printerror(status)
         endif
      endif
      status=0

! --> Determine the size of the image.
      call ftgknj(unit,'NAXIS',1,2,naxes,nfound,status)
      if(nfound.ne.2) stop '*** Unexpected number of axes found ***'

! --> Initialize variables for reading field inclination. 
      narray=naxes(1)*naxes(2)
      group=1
      nullval=-999.
      firstpix=1

! --> Read in field inclination.
      call ftgpve(unit,group,firstpix,narray,nullval,bi1d,anynull,status)

   elseif(hdutype.eq.2) then

! --> Check this is the field inclination.
      call ftgkys(unit,'EXTNAME',record,comment,status)
      if(record.ne.'FieldInclination') then
         write(*,*) 'unexpected EXTNAME for field inclination',record
      endif

! --> Determine the size of the extension.
      maxdim=2
      colnum=1
      call ftgtcl(unit,colnum,datacode,repeat,width,status)
      call printerror(status)

      call ftgtdm(unit,colnum,maxdim,naxis,naxes,status)
      call printerror(status)

      if(naxis.ne.2) then
         write(*,*) '*** Unexpected number of axes found while reading field inclination ***',naxis
         write(*,*) naxes(1),naxes(2)
      endif

      if(repeat.ne.naxes(1)*naxes(2)) then
         write(*,*) '*** Inconsistent row length found while reading field inclination ***'
         write(*,*) repeat,naxes(1),naxes(2)
      endif

! --> Initialize variables for reading field inclination. 
      irow=1
      felem=1
      nelems=repeat

! --> Read in field inclination.
      if(datacode.eq.42) then
         call ftgcve(unit,colnum,irow,felem,nelems,nullval,bi1d,anynull,status)
      else
         write(*,*) 'unexpected data type for field inclination',datacode
         stop
      endif

   else
      write(*,*) 'unrecognized HDU type for field inclination',hdutype
      stop
   endif

! --> Check the dimensions of this array match the previous dimensions.
   if(nx.ne.naxes(1).or.ny.ne.naxes(2)) then
      write(*,*) 'array dimensions do not match for field inclination'
      write(*,*) 'nx,ny',ny,ny
      write(*,*) 'naxes',naxes(1),naxes(2)
      stop
   endif

! --> Repeat for the HDU containing the field azimuth.

   call ftmahd(unit,jfa,hdutype,status)

! --> Check this HDU exists.

   if(status.ne.0) then
      write(*,*) 'error encountered while looking for field azimuth'
      call printerror(status)
      stop
   endif

! --> Have to treat this differently depending on whether it's an image 
! --> extension (hdutype=0) or a binary table extension (hdutype=2).

   if(hdutype.eq.0) then

! --> Check this is the field azimuth.
      call ftgkys(unit,'PAR_TYPE',record,comment,status)
      if(status.eq.0) then
         if(record.ne.'Field azimuth') then
            write(*,*) 'unexpected PAR_TYPE for field azimuth'
         else
            write(*,*) 'correct PAR_TYPE for field azimuth'
         endif
      else
         write(*,*) 'error encountered while getting field azimuth PAR_TYPE'
         call printerror(status)
         write(*,*) 'checking for EXTNAME instead'
         status=0
         call ftgkys(unit,'EXTNAME',record,comment,status)
         if(status.eq.0) then
            if(record.ne.'FieldAzimuth') then
               write(*,*) 'unexpected EXTNAME for field azimuth',record
            else
               write(*,*) 'correct EXTNAME found'
               write(*,*) ''
            endif
         else
            write(*,*) 'error encountered while getting field azimuth EXTNAME'
            call printerror(status)
         endif
      endif
      status=0

! --> Determine the size of the image.
      call ftgknj(unit,'NAXIS',1,2,naxes,nfound,status)
      if(nfound.ne.2) stop '*** Unexpected number of axes found while reading field azimuth ***'

! --> Initialize variables for reading field azimuth. 
      narray=naxes(1)*naxes(2)
      group=1
      nullval=-999.
      firstpix=1

! --> Read in field azimuth.
      call ftgpve(unit,group,firstpix,narray,nullval,ba1d,anynull,status)

   elseif(hdutype.eq.2) then

! --> Check this is the field azimuth.
      call ftgkys(unit,'EXTNAME',record,comment,status)
      if(record.ne.'FieldAzimuth') then
         write(*,*) 'unexpected EXTNAME for field azimuth',record
      endif

! --> Determine the size of the extension.
      maxdim=2
      colnum=1
      call ftgtcl(unit,colnum,datacode,repeat,width,status)
      call printerror(status)

      call ftgtdm(unit,colnum,maxdim,naxis,naxes,status)
      call printerror(status)

      if(naxis.ne.2) then
         write(*,*) '*** Unexpected number of axes found while reading field azimuth ***',naxis
         write(*,*) naxes(1),naxes(2)
      endif

      if(repeat.ne.naxes(1)*naxes(2)) then
         write(*,*) '*** Inconsistent row length found while reading field azimuth ***'
         write(*,*) repeat,naxes(1),naxes(2)
      endif

! --> Initialize variables for reading field azimuth. 
      irow=1
      felem=1
      nelems=repeat

! --> Read in field azimuth.
      if(datacode.eq.42) then
         call ftgcve(unit,colnum,irow,felem,nelems,nullval,ba1d,anynull,status)
      else
         write(*,*) 'unexpected data type for field azimuth',datacode
         stop
      endif

   else
      write(*,*) 'unrecognized HDU type for field azimuth',hdutype
      stop
   endif

! --> Check the dimensions of this array match the previous dimensions.
   if(nx.ne.naxes(1).or.ny.ne.naxes(2)) then
      write(*,*) 'array dimensions do not match for field azimuth'
       write(*,*) 'nx,ny',ny,ny
       write(*,*) 'naxes',naxes(1),naxes(2)
       stop
   endif

! --> Repeat for the HDU containing the straylight fraction.

   call ftmahd(unit,jsf,hdutype,status)

! --> Check this HDU exists.

   if(status.ne.0) then
      write(*,*) 'error encountered while looking for straylight fraction'
      call printerror(status)
      stop
   endif

! --> Have to treat this differently depending on whether it's an image 
! --> extension (hdutype=0) or a binary table extension (hdutype=2).

   if(hdutype.eq.0) then

! --> Check this is the straylight fraction.
      call ftgkys(unit,'PAR_TYPE',record,comment,status)
      if(status.eq.0) then
         if(record.ne.'Straylight fraction') then
            write(*,*) 'unexpected PAR_TYPE for Straylight fraction'
         else
            write(*,*) 'correct PAR_TYPE for Straylight fraction'
         endif
      else
         write(*,*) 'error encountered while getting Straylight fraction PAR_TYPE'
         call printerror(status)
         write(*,*) 'checking for EXTNAME instead'
         status=0
         call ftgkys(unit,'EXTNAME',record,comment,status)
         if(status.eq.0) then
            if(record.ne.'ScattLight') then
               write(*,*) 'unexpected EXTNAME for Straylight fraction',record
            else
               write(*,*) 'correct EXTNAME found'
               write(*,*) ''
            endif
         else
            write(*,*) 'error encountered while getting Straylight fraction EXTNAME'
            call printerror(status)
         endif
      endif
      status=0

! --> Determine the size of the image.
      call ftgknj(unit,'NAXIS',1,2,naxes,nfound,status)
      if(nfound.ne.2) stop '*** Unexpected number of axes found while reading Staylight fraction ***'

! --> Initialize variables for reading straylight fraction. 
      narray=naxes(1)*naxes(2)
      group=1
      nullval=-999.
      firstpix=1

! --> Read in straylight fraction.
      call ftgpve(unit,group,firstpix,narray,nullval,sl1d,anynull,status)

   elseif(hdutype.eq.2) then

! --> Check this is the stray light fill factor.
      call ftgkys(unit,'EXTNAME',record,comment,status)
      if(record.ne.'Alpha') then
         write(*,*) 'unexpected EXTNAME for Stray Light Fill Factor',record
      endif

! --> Determine the size of the extension.
      maxdim=2
      colnum=1
      call ftgtcl(unit,colnum,datacode,repeat,width,status)
      call printerror(status)

      call ftgtdm(unit,colnum,maxdim,naxis,naxes,status)
      call printerror(status)

      if(naxis.ne.2) then
         write(*,*) '*** Unexpected number of axes found while reading stray light fill factor ***',naxis
         write(*,*) naxes(1),naxes(2)
      endif

      if(repeat.ne.naxes(1)*naxes(2)) then
         write(*,*) '*** Inconsistent row length found while reading stray light fill factor ***'
         write(*,*) repeat,naxes(1),naxes(2)
      endif

! --> Initialize variables for reading stray light fill factor. 
      irow=1
      felem=1
      nelems=repeat

! --> Read in stray light fill factor.
      if(datacode.eq.42) then
         call ftgcve(unit,colnum,irow,felem,nelems,nullval,sl1d,anynull,status)
      else
         write(*,*) 'unexpected data type for stray light fill factor',datacode
         stop
      endif

   else
     write(*,*) 'unrecognized HDU type for Straylight fraction',hdutype
     stop
   endif

! --> Check the dimensions of this array match the previous dimensions.
   if(nx.ne.naxes(1).or.ny.ne.naxes(2)) then
      write(*,*) 'array dimensions do not match for straylight fraction'
      write(*,*) 'nx,ny',ny,ny
      write(*,*) 'naxes',naxes(1),naxes(2)
      stop
   endif

! --> Repeat for the HDU containing the continuum intensity.

   call ftmahd(unit,jci,hdutype,status)

! --> Check this HDU exists.

   if(status.ne.0) then
      write(*,*) 'error encountered while looking for continuum intensity'
      call printerror(status)
      stop
   endif

! --> Have to treat this differently depending on whether it's an image 
! --> extension (hdutype=0) or a binary table extension (hdutype=2).

   if(hdutype.eq.0) then

! --> Check this is the continuum intensity.
      call ftgkys(unit,'PAR_TYPE',record,comment,status)
      if(status.eq.0) then
         if(record.ne.'Continuum intensity') then
            write(*,*) 'unexpected PAR_TYPE for Continuum intensity'
         else
            write(*,*) 'correct PAR_TYPE for Continuum intensity'
         endif
      else
         write(*,*) 'error encountered while getting Continuum intensity PAR_TYPE'
         call printerror(status)
         write(*,*) 'checking for EXTNAME instead'
         status=0
         call ftgkys(unit,'EXTNAME',record,comment,status)
         if(status.eq.0) then
            if(record.ne.'Continuum') then
               write(*,*) 'unexpected EXTNAME for Continuum intensity',record
            else
               write(*,*) 'correct EXTNAME found'
               write(*,*) ''
            endif
         else
            write(*,*) 'error encountered while getting Continuum intensity EXTNAME'
            call printerror(status)
         endif
      endif
      status=0

! --> Determine the size of the image.
      call ftgknj(unit,'NAXIS',1,2,naxes,nfound,status)
      if(nfound.ne.2) stop '*** Unexpected number of axes found while reading continuum intensity ***'

! --> Initialize variables for reading continuum intensity. 
      narray=naxes(1)*naxes(2)
      group=1
      nullval=-999.
      firstpix=1

! --> Read in continuum intensity. 
      call ftgpve(unit,group,firstpix,narray,nullval,ci1d,anynull,status)

   elseif(hdutype.eq.2) then

! --> Check this is the line strength.
      call ftgkys(unit,'EXTNAME',record,comment,status)
! --> Note typo: this is what appears to be in the level 2 data.
      if(record.ne.'LineStength') then
         write(*,*) 'unexpected EXTNAME for Line Strength',record
      endif

! --> Determine the size of the extension.
      maxdim=2
      colnum=1
      call ftgtcl(unit,colnum,datacode,repeat,width,status)
      call printerror(status)

      call ftgtdm(unit,colnum,maxdim,naxis,naxes,status)
      call printerror(status)

      if(naxis.ne.2) then
         write(*,*) '*** Unexpected number of axes found while reading line strength ***',naxis
         write(*,*) naxes(1),naxes(2)
      endif

      if(repeat.ne.naxes(1)*naxes(2)) then
         write(*,*) '*** Inconsistent row length found while reading line strength ***'
         write(*,*) repeat,naxes(1),naxes(2)
      endif

! --> Initialize variables for reading line strength. 
      irow=1
      felem=1
      nelems=repeat

! --> Read in line strength.
      if(datacode.eq.42) then
         call ftgcve(unit,colnum,irow,felem,nelems,nullval,ci1d,anynull,status)
      else
         write(*,*) 'unexpected data type for line strength',datacode
         stop
      endif

   else
      write(*,*) 'unrecognized HDU type for Continuum intensity',hdutype
      stop
   endif

! --> Check the dimensions of this array match the previous dimensions.
   if(nx.ne.naxes(1).or.ny.ne.naxes(2)) then
      write(*,*) 'array dimensions do not match for continuum intensity'
      write(*,*) 'nx,ny',ny,ny
      write(*,*) 'naxes',naxes(1),naxes(2)
      stop
   endif

! --> Set a continuum intensity to define off-limb observations. 
! --> Default is less than 5% of the mean continuum intensity. 
   cimean=0.
   do j=1,nx*ny
      cimean=cimean+ci1d(j)
   enddo
   cimean=cifrac*cimean/float(nx*ny)

! --> Convert field strength to mean field ("flux") and zero the field
! --> strength off the limb.
   nlimb=0
   do j=1,nx*ny
      if(ci1d(j).gt.cimean) then
         bs1d(j)=bs1d(j)*sl1d(j)
         !bs1d(j)=bs1d(j)*(1.0-sl1d(j))
      else
         bs1d(j)=0.
         nlimb=nlimb+1
      endif
   enddo
!      if(nlimb.ne.0) write(*,*) nlimb,'pixels off limb'

   if(status.gt.0) call printerror(status)

   phase=0.5*float(iaflag)*pi

! --> Convert to radians if necessary. 
   if(iaunit.eq.1) then
      dtor=pi/180.
      do i=1,nx*ny
         bi1d(i)=bi1d(i)*dtor
         ba1d(i)=ba1d(i)*dtor
      enddo
   endif

! --> Transform to two components. 
   if(incflag.eq.0) then
      do i=1,nx
         do j=1,ny
            i1d=i+(j-1)*nx
            bl(i,j)=bs1d(i1d)*cos(bi1d(i1d))
            bt(i,j)=bs1d(i1d)*sin(bi1d(i1d))
            ba(i,j)=ba1d(i1d)
! --> Calculate Cartesian (image) components.
            Bx(i,j)=bt(i,j)*cos(ba(i,j)+phase)
            By(i,j)=bt(i,j)*sin(ba(i,j)+phase)
            Bz(i,j)=bl(i,j)
         enddo
      enddo
   else
      do i=1,nx
         do j=1,ny
            i1d=i+(j-1)*nx
            bl(i,j)=bs1d(i1d)*sin(bi1d(i1d))
            bt(i,j)=bs1d(i1d)*cos(bi1d(i1d))
            ba(i,j)=ba1d(i1d)
! --> Calculate Cartesian (image) components.
            Bx(i,j)=bt(i,j)*cos(ba(i,j)+phase)
            By(i,j)=bt(i,j)*sin(ba(i,j)+phase)
            Bz(i,j)=bl(i,j)
         enddo
      enddo
   endif

! --> The FITS file must always be closed before exiting the program. 
! --> Any unit numbers allocated with FTGIOU must be freed with FTFIOU.
   call ftclos(unit, status)
   call ftfiou(unit, status)

! --> Check for any error, and if so print out error messages.
   if(status.gt.0) call printerror(status)

! --> Deallocate memory.
   deallocate(bs1d,bi1d,ba1d,sl1d,ci1d)

end subroutine rd_sotsp
